# test_module::titles
#
# A description of what this class does
#
# @summary A short summary of the purpose of this class
#
# @example
#   include test_module::titles
class test_module::titles {
  title_provider { 'some title':
    namevar => 'some id'
  }
}
